use std::{io::Cursor, path::PathBuf};

fn find_subsequence(haystack: &[u8], needle: &[u8]) -> Vec<usize> {
    haystack.windows(needle.len())
        .enumerate()
        .filter_map(|(i, arr)| if arr == needle { Some(i) } else { None })
        .collect()
}

fn recursively_find_classes(path: PathBuf) -> Vec<PathBuf> {
    let e = std::fs::read_dir(path).unwrap();
    let mut classes = Vec::new();
    for entry in e {
        if let Ok(entry) = entry {
            match entry.metadata().unwrap().is_dir() {
                true => classes.append(&mut recursively_find_classes(entry.path())),
                false => {
                    if entry.file_name().to_str().unwrap().ends_with(".class") {
                        classes.push(entry.path());
                    }
                }
            }
        }
    }
    classes
}

fn main() {
    let mut target_dir = PathBuf::new();
    target_dir.push("out");

    //let archive = std::fs::read(r#"D:\Documents\RustroverProjects\XCraft\xcraft\libraries\com\mojang\authlib\1.5.22\authlib-1.5.22.jar"#).unwrap();

    //zip_extract::extract(Cursor::new(archive), &target_dir, true);

    for path in recursively_find_classes(PathBuf::from(".\\out")) {
        let haystack = std::fs::read(&path).unwrap();
        let v = find_subsequence(&haystack, b"mojang.com");
        if !v.is_empty() {
            println!("File {:?}: {:#?}",path, v);
        }
    }

    //let haystack = std::fs::read(r#".\out\com\mojang\authlib\yggdrasil\YggdrasilMinecraftSessionService.class"#).unwrap();

   // println!("Result: {:#?}", find_subsequence(&haystack, b"yggdrasil_session_pubkey.der"));
}